@@ -5,18 +5,25 @@
  *
  * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
  *
- * $Id: os.h,v 1.4 2001/12/18 08:30:32 dugsong Exp $
+ * $Id: os.h,v 1.6 2002/01/07 02:13:39 dugsong Exp $
  */
 
 #ifndef DNET_OS_H
 #define DNET_OS_H
 
-/* XXX - require POSIX */
-#include <sys/param.h>
-
-/* XXX - Linux <feature.h>, IRIX <sys/endian.h>, etc. */
-#include <netinet/in.h>
-#include <arpa/inet.h>
+#ifdef WIN32
+# include <windows.h>
+# include <winsock2.h>
+# include <stdint.h>
+#else
+# include <sys/param.h>
+# include <sys/types.h>
+# include <sys/socket.h>
+# include <netinet/in.h>
+# include <arpa/inet.h>
+# include <netdb.h>
+# include <inttypes.h>
+#endif
 
 #define DNET_LIL_ENDIAN		1234
 #define DNET_BIG_ENDIAN		4321
@@ -45,12 +52,6 @@
 #elif defined (_BIT_FIELDS_HTOL)
 # define DNET_BYTESEX		DNET_BIG_ENDIAN
 #endif
-#if defined(_SYS_INT_TYPES_H) || defined(__INTTYPES_INCLUDED)
-# define u_int64_t		uint64_t
-# define u_int32_t		uint32_t
-# define u_int16_t		uint16_t
-# define u_int8_t		uint8_t
-#endif
 
 /* Nastiness from old BIND code. */
 #ifndef DNET_BYTESEX
@@ -76,7 +77,7 @@
 #undef __END_DECLS
 #ifdef __cplusplus
 # define __BEGIN_DECLS	extern "C" {
-# define __END_DECLS	}
+# define __END_DECLS	} /* extern "C" */
 #else
 # define __BEGIN_DECLS
 # define __END_DECLS
