@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: eth-bsd.c,v 1.3 2001/12/09 15:49:05 dugsong Exp $
+ * $Id: eth-bsd.c,v 1.7 2002/01/20 21:23:27 dugsong Exp $
  */
 
 #include "config.h"
@@ -37,62 +37,55 @@ struct eth_handle {
 };
 
 eth_t *
-eth_open(char *device)
+eth_open(const char *device)
 {
 	struct ifreq ifr;
 	char file[32];
 	eth_t *e;
-	int i, fd = -1;
+	int i;
 
+	if ((e = calloc(1, sizeof(*e))) == NULL)
+		return (NULL);
+	
 	for (i = 0; i < 32; i++) {
 		snprintf(file, sizeof(file), "/dev/bpf%d", i);
-		fd = open(file, O_WRONLY);
-		if (fd != -1 || errno != EBUSY)
+		e->fd = open(file, O_WRONLY);
+		if (e->fd != -1 || errno != EBUSY)
 			break;
 	}
-	if (fd < 0)
+	if (e->fd < 0)
 		return (NULL);
 	
 	memset(&ifr, 0, sizeof(ifr));
 	strlcpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
 	
-	if (ioctl(fd, BIOCSETIF, (char *)&ifr) < 0) {
-		close(fd);
-		return (NULL);
-	}
+	if (ioctl(e->fd, BIOCSETIF, (char *)&ifr) < 0)
+		return (eth_close(e));
 #ifdef BIOCSHDRCMPLT
 	i = 1;
-	if (ioctl(fd, BIOCSHDRCMPLT, &i) < 0) {
-		close(fd);
-		return (NULL);
-	}
+	if (ioctl(e->fd, BIOCSHDRCMPLT, &i) < 0)
+		return (eth_close(e));
 #endif
-	if ((e = calloc(1, sizeof(*e))) == NULL) {
-		close(fd);
-		return (NULL);
-	}
-	e->fd = fd;
 	strlcpy(e->device, device, sizeof(e->device));
 	
 	return (e);
 }
 
-ssize_t
+size_t
 eth_send(eth_t *e, const void *buf, size_t len)
 {
-	return (write(e->fd, buf, len));
+	return ((ssize_t)write(e->fd, buf, len));
 }
 
-int
+eth_t *
 eth_close(eth_t *e)
 {
 	assert(e != NULL);
 
-	if (close(e->fd) < 0)
-		return (-1);
-	
+	if (e->fd > 0)
+		close(e->fd);
 	free(e);
-	return (0);
+	return (NULL);
 }
 
 #if defined(HAVE_SYS_SYSCTL_H) && defined(HAVE_ROUTE_RT_MSGHDR)
@@ -145,14 +138,14 @@ eth_get(eth_t *e, eth_addr_t *ea)
 int
 eth_get(eth_t *e, eth_addr_t *ea)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 #endif
 
 #if defined(SIOCSIFLLADDR)
 int
-eth_set(eth_t *e, eth_addr_t *ea)
+eth_set(eth_t *e, const eth_addr_t *ea)
 {
 	struct ifreq ifr;
 	struct addr ha;
@@ -169,9 +162,9 @@ eth_set(eth_t *e, eth_addr_t *ea)
 }
 #else
 int
-eth_set(eth_t *e, eth_addr_t *ea)
+eth_set(eth_t *e, const eth_addr_t *ea)
 {
-	errno = EOPNOTSUPP;
+	errno = ENOSYS;
 	return (-1);
 }
 #endif
