@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: fw-ipf.c,v 1.5 2001/12/29 21:06:41 dugsong Exp $
+ * $Id: fw-ipf.c,v 1.14 2002/01/20 21:23:27 dugsong Exp $
  */
 
 #include "config.h"
@@ -16,15 +16,14 @@
 
 #include <net/if.h>
 #include <netinet/in.h>
+#define ip_t	ipf_ip_t
 #ifdef HAVE_NETINET_IP_FIL_COMPAT_H
 # include <netinet/ip_fil_compat.h>
 #else
 # include <netinet/ip_compat.h>
 #endif
 #include <netinet/ip_fil.h>
-#ifdef IP6EQ
-#define HAVE_I6ADDR	1
-#endif
+#undef ip_t
 
 #include <assert.h>
 #include <errno.h>
@@ -36,12 +35,12 @@
 
 #define KMEM_NAME	"/dev/kmem"
 
-typedef u_int32_t ip_addr_t;
+#include "dnet.h"
 
-#include "dnet/eth.h"
-#include "dnet/addr.h"
-#include "dnet/tcp.h"
-#include "dnet/fw.h"
+#if !defined(fi_saddr) && !defined(fi_daddr)
+# define fi_saddr	fi_src.s_addr
+# define fi_daddr	fi_dst.s_addr
+#endif
 
 struct fw_handle {
 	int	fd;
@@ -49,49 +48,50 @@ struct fw_handle {
 };
 
 static void
-rule_to_ipf(struct fw_rule *rule, struct frentry *fr)
+rule_to_ipf(const struct fw_rule *rule, struct frentry *fr)
 {
 	memset(fr, 0, sizeof(*fr));
 
-	if (*rule->device != '\0') {
-		strlcpy(fr->fr_ifname, rule->device, IFNAMSIZ);
-		strlcpy(fr->fr_oifname, rule->device, IFNAMSIZ);
+	if (*rule->fw_device != '\0') {
+		strlcpy(fr->fr_ifname, rule->fw_device, IFNAMSIZ);
+		strlcpy(fr->fr_oifname, rule->fw_device, IFNAMSIZ);
 	}
-	if (rule->op == FW_OP_ALLOW)
+	if (rule->fw_op == FW_OP_ALLOW)
 		fr->fr_flags |= FR_PASS;
 	else
 		fr->fr_flags |= FR_BLOCK;
+
+	if (rule->fw_dir == FW_DIR_IN)
+		fr->fr_flags |= FR_INQUE;
+	else
+		fr->fr_flags |= FR_OUTQUE;
 	
-	fr->fr_ip.fi_p = rule->proto;
-#ifdef HAVE_IP6ADDR
-	fr->fr_ip.fi_saddr = rule->src.addr_ip;
-	fr->fr_ip.fi_daddr = rule->dst.addr_ip;
-	addr_btom(rule->src.addr_bits, &fr->fr_mip.fi_saddr, IP_ADDR_LEN);
-	addr_btom(rule->dst.addr_bits, &fr->fr_mip.fi_daddr, IP_ADDR_LEN);
-#else
-	fr->fr_ip.fi_src.s_addr = rule->src.addr_ip;
-	fr->fr_ip.fi_dst.s_addr = rule->dst.addr_ip;
-	addr_btom(rule->src.addr_bits, &fr->fr_mip.fi_src.s_addr, IP_ADDR_LEN);
-	addr_btom(rule->dst.addr_bits, &fr->fr_mip.fi_dst.s_addr, IP_ADDR_LEN);
-#endif
-	switch (rule->proto) {
+	fr->fr_ip.fi_p = rule->fw_proto;
+	fr->fr_ip.fi_saddr = rule->fw_src.addr_ip;
+	fr->fr_ip.fi_daddr = rule->fw_dst.addr_ip;
+	addr_btom(rule->fw_src.addr_bits, &fr->fr_mip.fi_saddr, IP_ADDR_LEN);
+	addr_btom(rule->fw_dst.addr_bits, &fr->fr_mip.fi_daddr, IP_ADDR_LEN);
+	
+	switch (rule->fw_proto) {
 	case IPPROTO_ICMP:
-		fr->fr_icmpm = rule->sport[1] << 8 | (rule->dport[1] & 0xff);
-		fr->fr_icmp = rule->sport[0] << 8 | (rule->dport[0] & 0xff);
+		fr->fr_icmpm = rule->fw_sport[1] << 8 |
+		    (rule->fw_dport[1] & 0xff);
+		fr->fr_icmp = rule->fw_sport[0] << 8 |
+		    (rule->fw_dport[0] & 0xff);
 		break;
 	case IPPROTO_TCP:
 	case IPPROTO_UDP:
-		fr->fr_sport = rule->sport[0];
-		if (rule->sport[0] != rule->sport[1]) {
+		fr->fr_sport = rule->fw_sport[0];
+		if (rule->fw_sport[0] != rule->fw_sport[1]) {
 			fr->fr_scmp = FR_INRANGE;
-			fr->fr_stop = rule->sport[1];
+			fr->fr_stop = rule->fw_sport[1];
 		} else
 			fr->fr_scmp = FR_EQUAL;
 
-		fr->fr_dport = rule->dport[0];
-		if (rule->dport[0] != rule->dport[1]) {
+		fr->fr_dport = rule->fw_dport[0];
+		if (rule->fw_dport[0] != rule->fw_dport[1]) {
 			fr->fr_dcmp = FR_INRANGE;
-			fr->fr_dtop = rule->dport[1];
+			fr->fr_dtop = rule->fw_dport[1];
 		} else
 			fr->fr_dcmp = FR_EQUAL;
 		break;
@@ -99,7 +99,7 @@ rule_to_ipf(struct fw_rule *rule, struct frentry *fr)
 }
 
 static void
-ipf_ports_to_rule(u_char cmp, u_short port, u_short top, u_short *range)
+ipf_ports_to_rule(uint8_t cmp, uint16_t port, uint16_t top, uint16_t *range)
 {
 	switch (cmp) {
 	case FR_EQUAL:
@@ -140,44 +140,36 @@ ipf_ports_to_rule(u_char cmp, u_short port, u_short top, u_short *range)
 }
 
 static void
-ipf_to_rule(struct frentry *fr, struct fw_rule *rule)
+ipf_to_rule(const struct frentry *fr, struct fw_rule *rule)
 {
 	memset(rule, 0, sizeof(*rule));
 
-	strlcpy(rule->device, fr->fr_ifname, sizeof(rule->device));
-	rule->op = (fr->fr_flags & FR_PASS) ? FW_OP_ALLOW : FW_OP_BLOCK;
-	rule->direction = (fr->fr_flags & FR_INQUE) ? FW_DIR_IN : FW_DIR_OUT;
-	rule->proto = fr->fr_ip.fi_p;
+	strlcpy(rule->fw_device, fr->fr_ifname, sizeof(rule->fw_device));
+	rule->fw_op = (fr->fr_flags & FR_PASS) ? FW_OP_ALLOW : FW_OP_BLOCK;
+	rule->fw_dir = (fr->fr_flags & FR_INQUE) ? FW_DIR_IN : FW_DIR_OUT;
+	rule->fw_proto = fr->fr_ip.fi_p;
 
-	rule->src.addr_type = rule->dst.addr_type = ADDR_TYPE_IP;
-#ifdef HAVE_I6ADDR
-	rule->src.addr_ip = fr->fr_ip.fi_saddr;
-	rule->dst.addr_ip = fr->fr_ip.fi_daddr;
+	rule->fw_src.addr_type = rule->fw_dst.addr_type = ADDR_TYPE_IP;
+	rule->fw_src.addr_ip = fr->fr_ip.fi_saddr;
+	rule->fw_dst.addr_ip = fr->fr_ip.fi_daddr;
 	addr_mtob(&fr->fr_mip.fi_saddr, IP_ADDR_LEN,
-	    &rule->src.addr_bits);
+	    &rule->fw_src.addr_bits);
 	addr_mtob(&fr->fr_mip.fi_daddr, IP_ADDR_LEN,
-	    &rule->dst.addr_bits);
-#else
-	rule->src.addr_ip = fr->fr_ip.fi_src.s_addr;
-	rule->dst.addr_ip = fr->fr_ip.fi_dst.s_addr;
-	addr_mtob(&fr->fr_mip.fi_src.s_addr, IP_ADDR_LEN,
-	    &rule->src.addr_bits);
-	addr_mtob(&fr->fr_mip.fi_dst.s_addr, IP_ADDR_LEN,
-	    &rule->dst.addr_bits);
-#endif
-	switch (rule->proto) {
+	    &rule->fw_dst.addr_bits);
+	
+	switch (rule->fw_proto) {
 	case IPPROTO_ICMP:
-		rule->sport[0] = ntohs(fr->fr_icmp & fr->fr_icmpm) >> 8;
-		rule->sport[1] = ntohs(fr->fr_icmpm) >> 8;
-		rule->dport[0] = ntohs(fr->fr_icmp & fr->fr_icmpm) & 0xff;
-		rule->dport[1] = ntohs(fr->fr_icmpm) & 0xff;
+		rule->fw_sport[0] = ntohs(fr->fr_icmp & fr->fr_icmpm) >> 8;
+		rule->fw_sport[1] = ntohs(fr->fr_icmpm) >> 8;
+		rule->fw_dport[0] = ntohs(fr->fr_icmp & fr->fr_icmpm) & 0xff;
+		rule->fw_dport[1] = ntohs(fr->fr_icmpm) & 0xff;
 		break;
 	case IPPROTO_TCP:
 	case IPPROTO_UDP:
 		ipf_ports_to_rule(fr->fr_scmp, fr->fr_sport,
-		    fr->fr_stop, rule->sport);
+		    fr->fr_stop, rule->fw_sport);
 		ipf_ports_to_rule(fr->fr_dcmp, fr->fr_dport,
-		    fr->fr_dtop, rule->dport);
+		    fr->fr_dtop, rule->fw_dport);
 		break;
 	}
 }
@@ -190,20 +182,17 @@ fw_open(void)
 	if ((fw = calloc(1, sizeof(*fw))) == NULL)
 		return (NULL);
 
-	if ((fw->fd = open(IPL_NAME, O_RDWR, 0)) < 0) {
-		free(fw);
-		return (NULL);
-	}
-	if ((fw->kfd = open(KMEM_NAME, O_RDONLY)) < 0) {
-		close(fw->fd);
-		free(fw);
-		return (NULL);
-	}
+	if ((fw->fd = open(IPL_NAME, O_RDWR, 0)) < 0)
+		return (fw_close(fw));
+	
+	if ((fw->kfd = open(KMEM_NAME, O_RDONLY)) < 0)
+		return (fw_close(fw));
+
 	return (fw);
 }
 
 int
-fw_add(fw_t *fw, struct fw_rule *rule)
+fw_add(fw_t *fw, const struct fw_rule *rule)
 {
 	struct frentry fr;
 	
@@ -215,7 +204,7 @@ fw_add(fw_t *fw, struct fw_rule *rule)
 }
 
 int
-fw_delete(fw_t *fw, struct fw_rule *rule)
+fw_delete(fw_t *fw, const struct fw_rule *rule)
 {
 	struct frentry fr;
 	
@@ -227,7 +216,7 @@ fw_delete(fw_t *fw, struct fw_rule *rule)
 }
 
 static int
-fw_kcopy(fw_t *fw, u_char *buf, off_t pos, ssize_t n)
+fw_kcopy(fw_t *fw, u_char *buf, off_t pos, size_t n)
 {
 	int i;
 	
@@ -260,14 +249,16 @@ fw_loop(fw_t *fw, fw_handler callback, void *arg)
 #endif
 		return (-1);
 
-	for (frp = fio.f_fout[(int)fio.f_active]; frp != NULL; frp = fr.fr_next) {
+	for (frp = fio.f_fout[(int)fio.f_active]; frp != NULL;
+	    frp = fr.fr_next) {
 		if (fw_kcopy(fw, (u_char *)&fr, (u_long)frp, sizeof(fr)) < 0)
 			return (-1);
 		ipf_to_rule(&fr, &rule);
 		if ((ret = callback(&rule, arg)) != 0)
 			return (ret);
 	}
-	for (frp = fio.f_fin[(int)fio.f_active]; frp != NULL; frp = fr.fr_next) {
+	for (frp = fio.f_fin[(int)fio.f_active]; frp != NULL;
+	    frp = fr.fr_next) {
 		if (fw_kcopy(fw, (u_char *)&fr, (u_long)frp, sizeof(fr)) < 0)
 			return (-1);
 		ipf_to_rule(&fr, &rule);
@@ -277,14 +268,15 @@ fw_loop(fw_t *fw, fw_handler callback, void *arg)
 	return (0);
 }
 
-int
+fw_t *
 fw_close(fw_t *fw)
 {
 	assert(fw != NULL);
 
-	if (close(fw->fd) < 0 || close(fw->kfd) < 0)
-		return (-1);
-	
+	if (fw->fd > 0)
+		close(fw->fd);
+	if (fw->kfd > 0)
+		close(fw->kfd);
 	free(fw);
-	return (0);
+	return (NULL);
 }
