@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: ip-cooked.c,v 1.9 2002/02/09 03:39:58 dugsong Exp $
+ * $Id: ip-cooked.c,v 1.11 2002/02/23 20:12:22 dugsong Exp $
  */
 
 #include "config.h"
@@ -47,14 +47,15 @@ _add_ip_intf(const struct intf_entry *entry, void *arg)
 	if (entry->intf_type == INTF_TYPE_ETH &&
 	    (entry->intf_flags & INTF_FLAG_UP) != 0 &&
 	    entry->intf_mtu >= ETH_LEN_MIN &&
-	    entry->intf_addr != NULL && entry->intf_link_addr != NULL) {
+	    entry->intf_addr.addr_type == ADDR_TYPE_IP &&
+	    entry->intf_link_addr.addr_type == ADDR_TYPE_ETH) {
 		
 		if ((ipi = calloc(1, sizeof(*ipi))) == NULL)
 			return (-1);
 		
 		strlcpy(ipi->name, entry->intf_name, sizeof(ipi->name));
-		memcpy(&ipi->ha, entry->intf_link_addr, sizeof(ipi->ha));
-		memcpy(&ipi->pa, entry->intf_addr, sizeof(ipi->pa));
+		memcpy(&ipi->ha, &entry->intf_link_addr, sizeof(ipi->ha));
+		memcpy(&ipi->pa, &entry->intf_addr, sizeof(ipi->pa));
 		ipi->mtu = entry->intf_mtu;
 
 		LIST_INSERT_HEAD(&ip->ip_intf_list, ipi, next);
@@ -199,9 +200,9 @@ ip_send(ip_t *ip, const void *buf, size_t len)
 			    ((p - start) >> 3));
 
 			ip_checksum(iph, ip_hl + fraglen);
-			
-			if (eth_send(ipi->eth, frame,
-			    ETH_HDR_LEN + ip_hl + fraglen) < 0)
+
+			i = ETH_HDR_LEN + ip_hl + fraglen;
+			if (eth_send(ipi->eth, frame, i) != i)
 				return (-1);
 			p += fraglen;
 			if (end - p < fraglen)
@@ -210,8 +211,8 @@ ip_send(ip_t *ip, const void *buf, size_t len)
 		return (len);
 	}
 	memcpy(frame + ETH_HDR_LEN, buf, len);
-	
-	if (eth_send(ipi->eth, frame, ETH_HDR_LEN + len) != ETH_HDR_LEN + len)
+	i = ETH_HDR_LEN + len;
+	if (eth_send(ipi->eth, frame, i) != i)
 		return (-1);
 	
 	return (len);
